/* KallistiOS 1.1.6

   sound.c
   taken from the KOS-1.16 source (c)2000 Dan Potter

*/

CVSID("$Id: s3m.c,v 1.2 2002/01/06 00:40:33 bardtx Exp $");



/* Var to check for channel position */
static vuint32 *cmd = (vuint32*)(0xa0810000);
static volatile aica_channel *chans = (volatile aica_channel*)(0xa0810004);


/* SPU RAM malloc pointer */
static uint32 ram_base, ram_top;

static char *streamdrv = NULL;
static int streamdrv_size;


typedef struct {
	uint32	loc, len;
	uint32	rate;
	uint32	used;
} snd_effect;

#define SFX_CNT 20
static snd_effect sfx[SFX_CNT] = { {0,0,0,0} };

static int sfx_find_open() {
	int i;
	for (i=0; i<SFX_CNT; i++)
		if (sfx[i].used == 0)
			return i;

	return -1;
}


uint32 sfx_load_sample(const uint16 *src, uint32 len) {
	uint32 where;

	where = ram_top;
	spu_memload(where, (uint8*)src, len);
	ram_top = (ram_top + len + 3) & (~3);
	return where;
}



/* "Kicks" a channel command */
static void chn_kick(int chn) {
	*cmd = AICA_CMD_KICK | (chn+1);
	spu_write_wait();
}




/* WAV header:
	0x08	-- "WAVE"
	0x16	-- short num channels (1/2)
	0x18	-- long  HZ
	0x22	-- short 8 or 16 (bits)
	0x28	-- long  data length
	0x2c	-- data start

 */


int sfx_load(const char *fn) {
	uint32	fd, len, hz;
	uint16	*tmp, chn, fmt;
	int	idx;

	idx = sfx_find_open();
	if (idx < 0) {
		printf("sndserver: no more sfx slots\r\n");
		return -1;
	}

	printf("sndserver: loading effect %s as slot %d\r\n", fn, idx);

	fd = fs_open(fn, O_RDONLY);
	if (fd == 0) {
		printf("sndserver: can't open sfx %s\r\n", fn);
		return -1;
	}

	/* Check file magic */
	hz = 0;
	fs_seek(fd, 8, SEEK_SET);
	fs_read(fd, &hz, 4);
	if (strncmp((char*)&hz, "WAVE", 4)) {
		printf("sndserver: file is not RIFF WAVE\r\n");
		fs_close(fd);
		return -1;
	}


	/* Read WAV header info */
	fs_seek(fd, 0x16, SEEK_SET);
	fs_read(fd, &chn, 2);
	fs_read(fd, &hz, 4);
	fs_seek(fd, 0x22, SEEK_SET);
	fs_read(fd, &fmt, 2);

//	 printf("WAVE file is %s, %dHZ, %d bits\r\n", chn==1 ? "mono" : "stereo", hz, fmt);

	/* Read WAV data */
	fs_seek(fd, 0x28, SEEK_SET);
	fs_read(fd, &len, 4);
	// printf("Reading %d bytes of wave data\r\n", len);
	tmp = malloc(len);
	fs_read(fd, tmp, len);
	fs_close(fd);
	sfx[idx].loc = sfx_load_sample(tmp, len);
	free(tmp);

	sfx[idx].len = len;
	sfx[idx].rate = hz;
	sfx[idx].used = 1;

	return idx;
}



void sfx_play_effect(int chn, uint32 src, uint32 freq, uint32 len, uint32 vol, uint32 pan) {
	chans[chn].cmd = AICA_CMD_START;
	chans[chn].pos = src;
	chans[chn].length = len;
	chans[chn].freq = freq;
	chans[chn].vol = vol;
	chans[chn].pan = pan;
	chn_kick(chn);
}


void play_sfx(int idx, int vol, int pan, global *g_point) {
	int size;

	size = sfx[idx].len/2;
	if (size > 65536) size = 65535;


/* printf("sndstream: playing effect %d on chan %d, loc %x, rate %d, size %d, vol %d, pan %d\r\n",
		idx, g_point->sfx_lastchan, sfx[idx].loc, sfx[idx].rate, size, vol, pan);
*/
	sfx_play_effect(g_point->sfx_lastchan, sfx[idx].loc, sfx[idx].rate, size, vol, pan);

	g_point->sfx_lastchan++;
	if (g_point->sfx_lastchan >= 64)
		g_point->sfx_lastchan = 0;
}

int pos2pan(float position)
	{
	int pan;
	if (position<-2.0f)
		pan=-16-((position-2)/2);
	else
		{
		if (position>2.0f)
			pan=((position-2)/2);
		else pan=0;
		}
	if (pan<-15)
		pan=-15;

	if (pan>15)
		pan=15;

return pan;
	}



